(function () {
  function hasClass(elem, classes) {
    if (typeof classes === 'string') {
      classes = [classes];
    }
    var elem_classes = elem.className.split(' ');

    for (var c = 0; c < classes.length; c++) {
      if (elem_classes.indexOf(classes[c]) < 0) {
        return false;
      }
    }

    return true;
  }

  function forSiblings(elem, func) {
    if (elem.parentElement && elem.parentElement.children && elem.parentElement.children.length) {
      for (var c = 0; c < elem.parentElement.children.length; c++) {
        if (elem.parentElement.children[c] !== elem) {
          func(elem.parentElement.children[c], c, elem.parentElement.children);
        }
      }
    }
  }

  function clickHandler (e) {
    var elem = e.target, heirarchy = [], trace = elem;

    while (trace) {
      heirarchy.push(trace);
      trace = trace.parentElement;
    }
  
    if (hasClass(elem, ['menu-link', 'has-submenu', 'menu-handler'])) {
      if (hasClass(elem, 'open')) {
        elem.classList.remove('open');

        if (elem.parentElement) {
          elem.parentElement.classList.remove('open');
        }

        forSiblings(elem, function (sib) {
          if (hasClass(sib, 'menu')) {
            sib.setAttribute('aria-hidden', true);
          }
        });
      } else {
        elem.classList.add('open');

        if (elem.parentElement) {
          elem.parentElement.classList.add('open');
        }
  
        forSiblings(elem, function (sib) {
          if (hasClass(sib, 'menu')) {
            sib.setAttribute('aria-hidden', false);
          }
        });
      }

      e.preventDefault();
      e.stopPropagation();
    } else {
      elem = null;
    }

    var elems = document.querySelectorAll('.dropdown-menus .menu-link.has-submenu.open');

    for (var c = 0; c < elems.length; c++) {
      if (heirarchy.indexOf(elems[c]) < 0) {
        elems[c].classList.remove('open');

        if (elems[c].parentElement) {
          elems[c].parentElement.classList.remove('open');
        }

        forSiblings(elems[c], function (sib) {
          if (hasClass(sib, 'menu')) {
            sib.setAttribute('aria-hidden', true);
          }
        });
      }
    }  
  }

  function keyupHandler (e) {
    if (e.keyCode === 27) {
      var elems = document.querySelectorAll('.dropdown-menus .menu-link.has-submenu.open');

      for (var c = 0; c < elems.length; c++) {
        elems[c].classList.remove('open');

        if (elems[c].parentElement) {
          elems[c].parentElement.classList.remove('open');
        }

        forSiblings(elems[c], function (sib) {
          if (hasClass(sib, 'menu')) {
            sib.setAttribute('aria-hidden', true);
          }
        });
      }

      if (elems.length > 0) {
        e.preventDefault();
        e.stopPropagation();
      }
    }
  }

  Drupal.behaviors.soahMenus = {
    attach: function () {
      var elems = document.querySelectorAll('.dropdown-menus .menu-link.has-submenu');

      for (var c = 0; c < elems.length; c++) {
        elems[c].classList.add('menu-handler');
        elems[c].setAttribute('aria-haspopup', true);

        forSiblings(elems[c], function (sib) {
          if (hasClass(sib, 'menu') && !sib.getAttribute('aria-hidden')) {
            sib.setAttribute('aria-hidden', true);
          }
        });
      }

    }
  };

  document.addEventListener('click', clickHandler);
  document.addEventListener('keyup', keyupHandler);
  Drupal.behaviors.soahMenus.attach();
})();
